#ifndef __COLOUR_H__
#define __COLOUR_H__



#ifdef _WINDOWS_
#include <windows.h>
#endif
#include <GL/gl.h>

#include <iostream>

class Colour
{ // holds an red,green,blue 3-tuple
public :
	/*!GLfloat red component of the colour tuple */
	GLfloat r;
	/*!GLfloat green component of the colour tuple */
	GLfloat g;
	/*!GLfloat blue component of the colour tuple */
	GLfloat b;
	/*!GLfloat  alpha the transparent element */
	GLfloat a;

	/*!default constructor set all values to 0 */
	  inline Colour(){r = g = b =  a= 0.0;}
	/*! constructor passing in r g b components
	 * \param float R red component
	 * \param float G green component
	 * \param float B blue component */
	  inline Colour(GLfloat R, GLfloat G, GLfloat B)
		{r = R; g = G; b = B;}
	  /*! constructor passing in r g b components
	 * \param float R red component
	 * \param float G green component
	 * \param float B blue component 
     * \param float A the alpha component
     */
     
	  inline Colour(GLfloat R, GLfloat G, GLfloat B, GLfloat A)
		{r = R; g = G; b = B;a=A;}

	/*! constructor passing in a Colour
	 * \Colour the colour to use
	 */
	  inline Colour(Colour const &c)
		{r = c.r; g = c.g; b = c.b; a=c.a;}

	/*! set method to change the current values of the colour
	 * \param float r red component to be set
	 * \param float g green component to be set
	 * \param float b blue component to be set
	 */
	inline void set(GLfloat R, GLfloat G, GLfloat B)
		{r = R; g = G; b = B; a=1.0;}
	/*! set method to change the current values of the colour
	 * as well as the transparency
	 * \param GLfloat r red component to be set
	 * \param GLfloat g green component to be set
	 * \param GLfloat b blue component to be set
	 * \param GLfloat alpha transparency value
	 */
	inline void set(GLfloat R, GLfloat G,GLfloat B, GLfloat A)
		{r=R; g=G; b=B; a=A;}
	/*! constructor passing in a Colour
	 * \Colour the colour to use
	 */
	inline void set(Colour c)
      {r = c.r; g = c.g; b= c.b; a=c.a;}

	Colour operator +(const Colour &rhs);
    void operator +=(const Colour &rhs);
    Colour operator *(const Colour &rhs);
    Colour operator *(GLfloat rhs);
    void operator *=(const Colour &rhs);
    void operator *=(GLfloat rhs);
	inline void add(Colour src, Colour refl);
	void build4tuple(GLfloat v[]);
/*! << extraction operator to prints out the colour in the format [r,g,b,a]
   \param std::ostream & output the outstream class pointer
   \param const Colour &s the colour to print out 
*/
friend std::ostream& operator<<(std::ostream& output, const Colour& s);
/*! >> insertion operator to read in the colour data r -> g -> b -> a
   \param std::istream & input the instream class pointer
   \param const Colour s the colour to read in to
*/
friend std::istream& operator>>(std::istream& input, Colour s);
/*! call glColor4f to set this as the current colour */
void Use(void);
};



#endif // colour
