#ifndef __MATRIX_H_
#define __MATRIX_H_

#include <string.h> // for memset
#include "Vector.h"
#include <GL/gl.h>
#include <iostream>
/*!Matrix Class to do simple matrix operations included 
 * operator overloaded functions for maths 
 * and matrix * vector type maths */
namespace GraphicsLib {
class Matrix
{
 friend std::ostream& operator<<(std::ostream& output, const Matrix& M);
public:
	/*! matrix element m as a 4x4 array */
	GLfloat m[4][4];
	Matrix();
    Matrix(const Matrix& M);
    Matrix(
    		GLfloat m00,GLfloat m01,GLfloat m02, GLfloat m03,
    		GLfloat m10,GLfloat m11,GLfloat m12, GLfloat m13,
            GLfloat m20,GLfloat m21,GLfloat m22, GLfloat m23,
            GLfloat m30,GLfloat m31,GLfloat m32, GLfloat m33
         );
	/*! Tofloat will convert the matrix to an all float
	 * value
	 * \param GLfloat in[16] the value to save to
	 */

	void Tofloat(GLfloat in[16]);
	/*!set the value at M[x][y] to equals
	 * \param GLint x the x index into the array
	 * \param GLint y the y index into the array
	 * \param GLfloat equals the value to set to
	 */
    void Mi(GLint x,GLint y,GLfloat equals);
	
	// clears matrix to all 0
	/*! clear the matrix to all 0 */
	void null(void);
	/*! make the matrix m the identity matrix <BR>
	 * 1 0 0 0 <BR>
	 * 0 1 0 0 <BR>
	 * 0 0 1 0 <BR>
	 * 0 0 0 1 <BR>
	 */
	// set as identity matrix
	void Identity(void);
	

Matrix operator*(const Matrix &m1);
void operator*= (const Matrix &m1);
Matrix operator+(const Matrix &m1);
void operator+=(const Matrix &m1);
Matrix operator*(const GLfloat i);
void operator*=(const GLfloat i);
void Transpose(void);
void LoadModelView(void);
void LoadProjection(void);
void RotateX(float deg);
void RotateY(float deg);
void RotateZ(float deg);
void Translate(GLfloat x, GLfloat y, GLfloat z);
void Scale(GLfloat x,GLfloat y, GLfloat z);
double Determinant(void);

};
}

#endif
