#ifndef __UTIL_H__
#define __UTIL_H__
#include "Point3.h"

#ifdef _WINDOWS_
	#include <windows.h>
#endif
#include <GL/gl.h>
namespace GraphicsLib
{
#define TWO_PI 6.28318530717958647692   //360
#define PI 3.14159265358979323846       //180
#define PI2 1.57079632679489661923      //90
#define PI4 0.785398163397448309615     //45

/*! Tween3 used to find the point t along two points
 * \param Point3 A the first point
 * \param Point3 B the second point
 * \param GLfloat the fraction along the line
 * \return Point3 the new point
 */
extern Point3 Tween3(Point3 A,Point3 B, GLfloat t);

/*! lerp simple linear interpolation funciton for 2 floats
   \param GLfloat A the first value
 * \param GLfloat B the second value
 * \param GLfloat the fraction along the line
 * \return GLfloat the new value
 */
extern GLfloat lerp(GLfloat A,GLfloat B, GLfloat t);

/*!DrawSquare draws a square centered at x,y of size/2
 * \param GLfloat x the x position
 * \param GLfloat y the y position
 * \param GLfloat size the size of the square
 */
extern void DrawSquare(GLfloat x, GLfloat y,GLfloat size);

/*!DrawAxis draws a 3d Axis at the current position
 * Weere the axis are represented as follows <BR>
X=RED
Y=GREEN
Z=BLUE
* \param GLfloat Size the scale factor for the axis
*/
extern void DrawAxis(GLfloat Size);


/*calculates the normal from 3 points and return the new
 * normal as a Vector
 * \param Point3 p1 the first point
 * \param Point3 p2 the second point
 * \param Point3 p4 the third point
 * \return Vector the normal of the 3 points
 */


extern Vector CalcNormal( Point3 p1, Point3 p2, Point3 p3 );
/*!converts Degrees to Radians
 * \param GLfloat deg the angle to convert
 * \param the angle in Radians
 */
extern GLfloat DegtoRad(GLfloat deg);
/*! converts Radians to Degrees
 * \param GLfloat rad the angle in radians
 * \returns the angle in Degrees
 */
extern GLfloat RadtoDeg(GLfloat rad);

extern void DrawAxisSimple(GLfloat Size);
extern float RandomNum();
extern float RandomNum(float MaxVal);
extern float RandomPosNum(float MaxVal);
extern void SetSeed(void);
}
#endif



