#include "stdafx.h"
#include "Vector.h"
#pragma pack(16)
using namespace GraphicsLib;
using namespace std;


void Vector::Normal(void)
{
	glNormal3f(x,y,z);
}

void Vector::Vertex(void)
{
	glVertex3f(x,y,z);
}

void Vector::Translate(void)
{
	glTranslatef(x,y,z);
}

GLfloat Vector::dot(Vector b) // return this dotted with b
{
	return x * b.x + y * b.y + z * b.z;
}

void  Vector::set(GLfloat xx, GLfloat yy, GLfloat zz)
{
	x=xx; y=yy; z=zz;
}

void Vector::set(GLfloat xx, GLfloat yy, GLfloat zz, GLfloat ww)
{
   x=xx; y=yy; z=zz; w=ww;
}

void Vector::set(Vector vin)
{
   x=vin.x; y=vin.y; z=vin.z;w=vin.w;
}
	// initialize from 3 or 4 floats
Vector::Vector(GLfloat xx,GLfloat yy,GLfloat zz,GLfloat ww)
{ 
   x=xx; y=yy; z=zz; w=ww; 
}

void Vector::null(void)
{ 
   x=y=z=0.0f; w=1.0f; 
}
// indexed component
GLfloat& Vector::operator[](GLint i) 
{ 
  return (&x)[i]; 
}

GLfloat Vector::length(void)	
{ 
  return (GLfloat)sqrt(x*x+y*y+z*z); 
}

void Vector::negate(void)
{ 
	x=-x; y=-y; z=-z; 
}
// make vector length one 
void Vector::normalize(void)
{
	GLfloat len=(GLfloat)sqrt(x*x+y*y+z*z);
	if (len==0.0f)
		return;
	x/=len; y/=len; z/=len;
}

void Vector::cross(const Vector& v1,const Vector& v2)
{
	x=v1.y*v2.z-v1.z*v2.y;
 	y=v1.z*v2.x-v1.x*v2.z;
 	z=v1.x*v2.y-v1.y*v2.x;
 	w=1.0f;
}

Vector Vector::cross(const Vector b) //return this cross b
{
	Vector c(y*b.z - z*b.y, z*b.x - x*b.z, x*b.y - y*b.x);
	return c;
}

void Vector::operator+=(const Vector& v)
{
    x+=v.x; y+=v.y; z+=v.z; w+=v.w;
}
void Vector::operator-=(const Vector& v)
{
	x-=v.x; y-=v.y; z-=v.z; w-=v.w;
}

void Vector::operator*=(const Vector& v)
{
	x*=v.x; y*=v.y; z*=v.z; w*=v.w;
}


void Vector::operator/=(const Vector& v)
{
	x/=v.x; y/=v.y; z/=v.z; w/=v.w;
}

Vector Vector::operator+(const Vector &v)
{
 Vector ret;
 ret.x=x+v.x; ret.y=y+v.y; ret.z=z+v.z; ret.w=w+v.w;
 return ret;
}



Vector Vector::operator-(const Vector& v)
{
	Vector ret;
	ret.x=x-v.x; ret.y=y-v.y; ret.z=z-v.z; ret.w=w-v.w;
	return ret;
}

bool Vector::operator==(const Vector& v)
{
if((v.x == x ) && (v.y == y) && (v.z == z) )
	return true;
else return false;
}



Vector Vector::operator*(const Vector &v)
{
	Vector ret;
	ret.x=x*v.x; ret.y=y*v.y; ret.z=z*v.z; ret.w=w*v.w;
	return ret;
}

Vector Vector::operator/(const Vector& v)
{
 Vector ret;
 ret.x=x/v.x; ret.y=y/v.y; ret.z=z/v.z; ret.w=w/v.w;
  return ret;
}

Vector Vector::operator *(GLfloat i)
{
 Vector ret;
 ret.x=x*i;
 ret.y=y*i;
 ret.z=z*i;
 ret.w=w*i;
 return ret;
}

GLdouble Vector::AngleBetween(const Vector &rhs)const 
{
   // uses cos(t) = a' . b'
  Vector v1 = rhs;
  Vector v2 = *this;
  v1.normalize();
  v2.normalize();
  return acos(v1.dot(v2));
}

Vector Vector::Reverse()
{
   x = -x;
   y = -y;
   z = -z;
   w = -w;
   return *this;
}

double Vector::inner(const Vector& vector) 
{
 return (x * vector.x) + (y * vector.y) + (z * vector.z);
}

Vector Vector::outer(const Vector& vector) 
{
  double xx = (y * vector.z) - (z * vector.y);
  double yy = (z * vector.x) - (x * vector.z);
  double zz = (x * vector.y) - (y * vector.x);

  return Vector(xx,yy,zz);
}

ostream& GraphicsLib::operator<<(ostream& output, const Vector& s)
  {
    return output<<"["<<s.x<<","<<s.y<<","<<s.z<<","<<s.w<<"]";
  }

Vector Vector::operator*(const Matrix &m)
	{
	  Vector a;
		float *f=(float *)&m;
		a.x = x*f[0] + y*f[4] + z*f[8] + w*f[12];
		a.y = x*f[1] + y*f[5] + z*f[9] + w*f[13];
		a.z = x*f[2] + y*f[6] + z*f[10] + w*f[14];
		a.w = x*f[3] + y*f[7] + z*f[11] + w*f[15];
	return a;
	}


