/* @file	Win32_Frametimer.c
 * @author	Rob
 * @brief	this file defines a windows32 frametimer using the high resolution
 *			frame counter available on all 0x86 processors.
 */
#include "stdafx.h"

#ifdef WIN32
#include <windows.h>
#include <time.h>
#include "Win32_Frametimer.h"

float	g_fStartOfFrame_,	/* the time (in secs) at the start of the frame */
		g_fEndOfFrame_  ,	/* the time (in secs) at the end of the frame */
	    g_frametime     ,	/* the total time taken to draw the frame */
		g_fRateInv_     ;	/* (1 / Processor frequency). This is an optimisation... */
__int64 StartClk_       ;	/* what was the time when the app started? */

/*
 * Quick func to return the current frametime
 */
float GetFrameTime()
{
	return g_frametime;
}

/*
 *	Call this function to Initialise the frame counter
 */
void  InitFrameTime()
{
	__int64 Rate;
	g_fRateInv_ = 1.0f/(float)CLOCKS_PER_SEC;

	/* see how quick the processor is going.... */
	if( !QueryPerformanceFrequency( ( LARGE_INTEGER* )&Rate) )
		return;

	/* err, if it appears to be dead... */
	if(!Rate)
		return;

	/* calculate 1/Rate and get hold of the start time... */
	g_fRateInv_=1.0f/(float)Rate;
	if(!QueryPerformanceCounter( ( LARGE_INTEGER* )&StartClk_) )
		return;
}

/*
 *	Get the current time
 */
float CurrentTime_()
{
	__int64 EndClk;
	/* see what time the processor is reporting... */
	QueryPerformanceCounter(( LARGE_INTEGER* )&EndClk);
	return  ( EndClk - StartClk_ )*g_fRateInv_;
}

/*
 *	Update frame time
 */
void  UpdateFrameTime()
{
	do
	{
		g_fEndOfFrame_=CurrentTime_();
	}
	while(g_fEndOfFrame_==g_fStartOfFrame_);

	g_frametime       =  g_fEndOfFrame_ - g_fStartOfFrame_ ;
	g_fStartOfFrame_  =  g_fEndOfFrame_ ;
}

#endif