/**
 * @file	Win32_Frametimer.h
 * @author	Rob
 * @brief	this file defines a windows32 frametimer using the high resolution
 *			frame counter available on all 0x86 processors.
 */

/*
 *	Sentry guard to prevent this file being included more than once
 */
#ifndef _WIN32_FRAMETIMER_H_
#define _WIN32_FRAMETIMER_H_

/*
 *	the 'extern' keyword is used in header files to say......
 */
#define DEFINED_IN_C_FILE extern


/**
 *	@brief	this function can be called to return the time the last frame
 *			took to draw in seconds
 *	@return	the time in seconds elapsed since the last frame was drawn
 */
DEFINED_IN_C_FILE float GetFrameTime();

/**
 *	@brief	this function should be called once at the start of the app to
 *			initialise the frame-counter
 */
DEFINED_IN_C_FILE void  InitFrameTime();

/**
 *	@brief	this function should be called once at the start of the app to
 *			initialise the frame-counter
 */
DEFINED_IN_C_FILE void  UpdateFrameTime();



#undef DEFINED_IN_C_FILE
#endif