/*
 * gltt graphics library
 * Copyright (C) 1998-1999 Stephane Rehel
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "stdafx.h"
#include "GLTTOutlineFont.h"

#include "FTFont.h"
#include "FTInstance.h"
#include "FTGlyphVectorizer.h"
#include "../myLogo.h"
#include "../MATRIX.H"

#include<vector>

#ifdef WIN32
#include <windows.h>
#endif

#ifdef LOCAL_GL_HEADER
  #include "local_gl.h"
#else
  #include <GL/gl.h>
#endif

using namespace std;
using namespace GraphicsLib;

vector <FTGlyphVectorizer*> vectorizerList;

extern  vector<CmyLogo*> CmyLogoList;

extern currentLogo;

/////////////////////////////////////////////////////////////////////////////
GLTTOutlineFont::GLTTOutlineFont() 
{
face= 0;

  instance= 0;
  font= 0;

  loaded= 0;
  list_base= 0;

  precision= 4.;
}


GLTTOutlineFont::GLTTOutlineFont( FTFace* _face )
{
  face= _face;

  instance= 0;
  font= 0;

  loaded= 0;
  list_base= 0;

  precision= 4.;
}

/////////////////////////////////////////////////////////////////////////////

GLTTOutlineFont::~GLTTOutlineFont()
{
  destroy();

  face= 0;
}

/////////////////////////////////////////////////////////////////////////////
void GLTTOutlineFont::setFace(FTFace* _face)
{
	face=_face;
}


void GLTTOutlineFont::destroy()
{
  delete[] loaded;
  loaded= 0;
  
  delete font;
  font= 0;

  delete instance;
  instance= 0;
}

/////////////////////////////////////////////////////////////////////////////

void GLTTOutlineFont::setPrecision( double _precision )
{
  precision= _precision;
}

/////////////////////////////////////////////////////////////////////////////

GLTTboolean GLTTOutlineFont::create( int point_size )
{
  destroy();

  if( point_size < 1 )
    point_size= 1;

  instance= new FTInstance(face);

  if( ! instance->create() )
    return GLTT_FALSE;

  int resolution= 96;
  if( ! instance->setResolutions(resolution,resolution) )
    return GLTT_FALSE;

  if( ! instance->setPointSize(point_size) )
    return GLTT_FALSE;

  int pixel_size= resolution * point_size / 72;

  font= new FTFont(instance);

  if( ! font->create() )
    return GLTT_FALSE;

  //list_base= glGenLists(256);
  //if( list_base == 0 )
  //  return GLTT_FALSE;

  loaded= new GLTTboolean [ 256 ];
  for( int i= 0; i < 256; ++i )
    loaded[i]= GLTT_FALSE;

  return GLTT_TRUE;
}

/////////////////////////////////////////////////////////////////////////////////////////////

GLTTboolean GLTTOutlineFont::loadGlyph( int i , int LetterCount)		//KGB added glyph loader
{
  if( i < 0 || i > 256 )
    return GLTT_FALSE;
  if( font == 0 )
    return GLTT_FALSE;

	//resize list of vectors to match string length
	if ( vectorizerList.size() > CmyLogoList[currentLogo]->stringName.GetLength())
	{
	  vectorizerList.resize(CmyLogoList[currentLogo]->stringName.GetLength());
	}

	while ( vectorizerList.size() < CmyLogoList[currentLogo]->stringName.GetLength())
	{
	  FTGlyphVectorizer* vectorizer = new FTGlyphVectorizer;
	  vectorizerList.push_back(vectorizer);
	}
    
  vectorizerList[LetterCount]->setPrecision(precision);	//set precision
  
  FTGlyph* glyph= font->getGlyph(i);
  if( glyph == 0 )
    {
    err:
    return GLTT_TRUE;
    }

  if( ! vectorizerList[LetterCount]->init(glyph) )
    goto err;

  vectorizerList[LetterCount]->vectorize();
  
	//for each letter in the logo string
   for( int j= 0; j < vectorizerList[LetterCount]->getNContours(); ++j )
   {
		FTGlyphVectorizer::Contour* contour = vectorizerList[LetterCount]->getContour(j);
		if( contour == 0 )
		continue;

		if( contour->nPoints <= 0 )
		continue;


		glBegin(GL_LINE_LOOP);

		for( int k= 0; k < contour->nPoints; ++k )
		{
			FTGlyphVectorizer::POINT& p= contour->points[k];//heres the point data  
			
			//p.y*= CmyLogoList[0]->height;					//set the y scale
			p.x+=getAdvance;								//offset subsequent letters	
			p.z=0;
		
		}

		glEnd();
		
    }
   
	getAdvance+= vectorizerList[LetterCount]->getAdvance();	//advance offset counter

  return GLTT_TRUE;
}
///////////////////////////////////////////////////////////////////////////////////////////////


GLTTboolean GLTTOutlineFont::loadGlyph( int i )
{/*
  if( i < 0 || i > 256 )
    return GLTT_FALSE;
  if( font == 0 )
    return GLTT_FALSE;


  FTGlyphVectorizer vectorizer;

  vectorizer.setPrecision(precision);
  
  FTGlyph* glyph= font->getGlyph(i);
  if( glyph == 0 )
    {
    err:
    return GLTT_TRUE;
    }

  if( ! vectorizer.init(glyph) )
    goto err;

  vectorizer.vectorize();

  for( int j= 0; j < vectorizer.getNContours(); ++j )
    {
    FTGlyphVectorizer::Contour* contour= vectorizer.getContour(j);
    if( contour == 0 )
      continue;

    if( contour->nPoints <= 0 )
      continue;


    glBegin(GL_LINE_LOOP);

    for( int k= 0; k < contour->nPoints; ++k )
      {
      FTGlyphVectorizer::POINT& p= contour->points[k];		//heres the point class
	
		glVertex3f( p.x, p.y, 0.0 );	// multiply p.x and y for scale deformations.
		
      }

    glEnd();

	glBegin(GL_POINTS);
		glVertex2f(centreX,centreY);
	glEnd();
    }
*/
  return GLTT_TRUE;
}



void GLTTOutlineFont::load( int from /* = 0 */, int to /* = 255 */ )
{
  for( int i= from; i <= to; ++i )
    loadGlyph(i);
}

/////////////////////////////////////////////////////////////////////////////

void GLTTOutlineFont::load( const char* text )
{
  if( text == 0 || list_base == 0 )
    return;

  for(;;)
    {
    int ch= (unsigned char)*text;
    if( ch == 0 )
      break;
    ++text;

    if( ! loaded[ch] )
      loadGlyph(ch);
    }
}

/////////////////////////////////////////////////////////////////////////////

void GLTTOutlineFont::output( const char* text  )
{
  if( text == 0  )
    return;

  int LetterCount =0;

  getAdvance=0;
  
  glPushMatrix();

  for(;;)
    {
    int ch= (unsigned char)*text;
    if( ch == 0 )
      break;
    ++text;
      loadGlyph(ch,LetterCount);
	  LetterCount ++;

    }
  glPopMatrix();
  centralize();
  Rotate();
  Draw();
}

/////////////////////////////////////////////////////////////////////////////

int GLTTOutlineFont::getHeight() const
{
  if( font == 0 )
    return 0;

  return font->getHeight();
}

/////////////////////////////////////////////////////////////////////////////

int GLTTOutlineFont::getDescender() const
{
  if( font == 0 )
    return 0;

  return font->getDescender();
}

/////////////////////////////////////////////////////////////////////////////

int GLTTOutlineFont::getWidth( const char* text )
{
  if( font == 0 )
    return 0;

  return font->getWidth(text);
}

/////////////////////////////////////////////////////////////////////////////

void GLTTOutlineFont::centralize()	//KGB added function to find and move text string to centre					
{
	double totalX=0;
	double totalY=0;
	double pointCount=0;
	
	for( int i= 0; i < vectorizerList.size(); i++ )					// for all letters
	{
		for( int j= 0; j < vectorizerList[i]->getNContours(); ++j )	// for all lines in letter
		{
			FTGlyphVectorizer::Contour* contour= vectorizerList[i]->getContour(j);
			if( contour == 0 )
			  continue;

			if( contour->nPoints <= 0 )
			  continue;

			for( int k= 0; k < contour->nPoints; ++k )				// for all points in the letter
			{
				FTGlyphVectorizer::POINT& p= contour->points[k];	//heres the point class

				
				totalX+=p.x;
				totalY+=p.y;
				pointCount++;
											
			}
		}	
	}
	totalX = totalX/pointCount;	//find centre of letters
	totalY = totalY/pointCount;	//find centre of letters

	double dispX=360-totalX;	//amount to move letters to be in the centre
	double dispY=288-totalY;	//amount to move letters to be in the centre

	for(  i= 0; i < vectorizerList.size(); i++ )
	{
		for(int  j= 0; j < vectorizerList[i]->getNContours(); ++j)
		{
			FTGlyphVectorizer::Contour* contour= vectorizerList[i]->getContour(j);
			if( contour == 0 )
			  continue;

			if( contour->nPoints <= 0 )
			  continue;

			

			for( int k= 0; k < contour->nPoints; ++k )
			{
				FTGlyphVectorizer::POINT& p= contour->points[k];		//heres the point class
			
				p.x+=dispX;
				p.y+=dispY;	
			}

			
		}	
	}
}


void GLTTOutlineFont::Rotate()
{
	Matrix myMatrix;

	for(int  i= 0; i < vectorizerList.size(); i++ )
	{
		for(int  j= 0; j < vectorizerList[i]->getNContours(); ++j)
		{
			FTGlyphVectorizer::Contour* contour= vectorizerList[i]->getContour(j);
			if( contour == 0 )
			  continue;

			if( contour->nPoints <= 0 )
			  continue;

			for( int k= 0; k < contour->nPoints; ++k )
			{
			FTGlyphVectorizer::POINT& p= contour->points[k];		//heres the point class

			Vector pointVector(	p.x-360-CmyLogoList[currentLogo]->PivotX,
								p.y-288-CmyLogoList[currentLogo]->PivotY,
								p.z-CmyLogoList[currentLogo]->PivotZ); 


			myMatrix.Identity();

			myMatrix.Scale(	CmyLogoList[currentLogo]->scale,
							CmyLogoList[currentLogo]->scale+CmyLogoList[0]->height,
							CmyLogoList[currentLogo]->scale);

			pointVector=pointVector*myMatrix;
			
			myMatrix.RotateX(CmyLogoList[currentLogo]->spinDegX);

			pointVector=pointVector*myMatrix;

			myMatrix.Identity();

			myMatrix.RotateY(CmyLogoList[currentLogo]->spinDegY);
			
			pointVector=pointVector*myMatrix;

			myMatrix.Identity();

			myMatrix.RotateZ(CmyLogoList[currentLogo]->spinDegZ);

			pointVector=pointVector*myMatrix;
			
			p.x =pointVector.x+360+CmyLogoList[currentLogo]->PivotX;
			p.y =pointVector.y+288+CmyLogoList[currentLogo]->PivotY;
			p.z =pointVector.z+CmyLogoList[currentLogo]->PivotZ;
	
			}
		}	
	}
}

void GLTTOutlineFont::Draw()
{

	for(int  i= 0; i < vectorizerList.size(); i++ )
	{
		for(int  j= 0; j < vectorizerList[i]->getNContours(); ++j)
		{
			FTGlyphVectorizer::Contour* contour= vectorizerList[i]->getContour(j);
			if( contour == 0 )
			  continue;

			if( contour->nPoints <= 0 )
			  continue;
			
			glBegin(GL_LINE_LOOP);

			for( int k= 0; k < contour->nPoints; ++k )
			{
			FTGlyphVectorizer::POINT& p= contour->points[k];		//heres the point class

			glVertex3f(p.x, p.y, p.z );

			
			}

			glEnd();

			
		}	
	}
}